@@ -306,7 +306,10 @@ tracker_property_get_domain (TrackerProperty *property)
 	priv = GET_PRIV (property);
 
 	if (!priv->domain && priv->use_gvdb) {
-		priv->domain = tracker_ontologies_get_class_by_uri (tracker_ontologies_get_property_string_gvdb (priv->uri, "domain"));
+		const gchar *domain_uri;
+
+		domain_uri = tracker_ontologies_get_property_string_gvdb (priv->uri, "domain");
+		priv->domain = g_object_ref (tracker_ontologies_get_class_by_uri (domain_uri));
 	}
 
 	return priv->domain;
@@ -338,12 +341,11 @@ tracker_property_get_domain_indexes (TrackerProperty *property)
 			g_variant_iter_init (&iter, variant);
 			while (g_variant_iter_loop (&iter, "s", &uri)) {
 				domain_index = tracker_ontologies_get_class_by_uri (uri);
-				g_free (uri);
 
 				tracker_property_add_domain_index (property, domain_index);
-
-				g_object_unref (domain_index);
 			}
+
+			g_variant_unref (variant);
 		}
 	}
 
@@ -360,7 +362,10 @@ tracker_property_get_range (TrackerProperty *property)
 	priv = GET_PRIV (property);
 
 	if (!priv->range && priv->use_gvdb) {
-		priv->range = tracker_ontologies_get_class_by_uri (tracker_ontologies_get_property_string_gvdb (priv->uri, "range"));
+		const gchar *range_uri;
+
+		range_uri = tracker_ontologies_get_property_string_gvdb (priv->uri, "range");
+		priv->range = g_object_ref (tracker_ontologies_get_class_by_uri (range_uri));
 	}
 
 	return priv->range;
