@@ -1,18 +1,18 @@
 /*
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  *
@@ -86,6 +86,14 @@ tracker_albumart_buffer_to_jpeg (const unsigned char *buffer,
 
 		pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 
+		if (pixbuf == NULL) {
+			g_warning ("Could not get pixbuf from GdkPixbufLoader when setting album art");
+
+			gdk_pixbuf_loader_close (loader, NULL);
+
+			return FALSE;
+		}
+
 		if (!gdk_pixbuf_save (pixbuf, target, "jpeg", &error, NULL)) {
 			g_warning ("Could not save GdkPixbuf when setting album art, %s",
 			           error ? error->message : "no error given");
