@@ -2,18 +2,18 @@
  * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
  * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
@@ -241,10 +241,6 @@ log_handler (const gchar    *domain,
              const gchar    *message,
              gpointer        user_data)
 {
-	if (!tracker_log_should_handle (log_level, verbosity)) {
-		return;
-	}
-
 	switch (log_level) {
 	case G_LOG_LEVEL_WARNING:
 	case G_LOG_LEVEL_CRITICAL:
@@ -278,14 +274,8 @@ run_standalone (void)
 	TrackerExtract *object;
 	GFile *file;
 	gchar *uri;
-	guint log_handler_id;
 
 	/* Set log handler for library messages */
-	log_handler_id = g_log_set_handler (NULL,
-	                                    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL,
-	                                    log_handler,
-	                                    NULL);
-
 	g_log_set_default_handler (log_handler, NULL);
 
 	/* Set the default verbosity if unset */
@@ -318,11 +308,6 @@ run_standalone (void)
 	g_object_unref (file);
 	g_free (uri);
 
-	if (log_handler_id != 0) {
-		/* Unset log handler */
-		g_log_remove_handler (NULL, log_handler_id);
-	}
-
 	tracker_albumart_shutdown ();
 
 	return EXIT_SUCCESS;
@@ -455,10 +440,8 @@ main (int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
-#ifdef HAVE_DBUS_FD_PASSING
 	tracker_dbus_connection_add_filter (tracker_extract_connection_filter,
 	                                    object);
-#endif
 
 	g_message ("Waiting for D-Bus requests...");
 
