@@ -45,7 +45,7 @@ grab_ungrab_with_ignorable_modifiers (GdkWindow *rootwin,
 			XGrabKey (GDK_WINDOW_XDISPLAY (rootwin),
 			          binding->keycode,
 			          binding->modifiers | mod_masks [i],
-			          GDK_WINDOW_XWINDOW (rootwin),
+			          GDK_WINDOW_XID (rootwin),
 			          False,
 			          GrabModeAsync,
 			          GrabModeAsync);
@@ -53,7 +53,7 @@ grab_ungrab_with_ignorable_modifiers (GdkWindow *rootwin,
 			XUngrabKey (GDK_WINDOW_XDISPLAY (rootwin),
 			            binding->keycode,
 			            binding->modifiers | mod_masks [i],
-			            GDK_WINDOW_XWINDOW (rootwin));
+			            GDK_WINDOW_XID (rootwin));
 		}
 	}
 }
