@@ -76,10 +76,7 @@ typedef struct {
 	GMainLoop *main_loop;
 	gchar *log_filename;
 
-	gchar *ttl_backup_file;
-
 	gboolean first_time_index;
-	gboolean reindex_on_shutdown;
 	gboolean shutdown;
 } TrackerMainPrivate;
 
@@ -122,7 +119,6 @@ private_free (gpointer data)
 
 	private = data;
 
-	g_free (private->ttl_backup_file);
 	g_free (private->log_filename);
 
 	g_main_loop_unref (private->main_loop);
@@ -225,44 +221,6 @@ initialize_priority (void)
 	 */
 }
 
-static void
-initialize_directories (void)
-{
-	/* NOTE: We don't create the database directories here, the
-	 * tracker-db-manager does that for us.
-	 */
-}
-
-static void
-shutdown_databases (void)
-{
-#if 0
-	TrackerMainPrivate *private;
-
-	private = g_static_private_get (&private_key);
-
-	/* TODO port backup support */
-
-	/* If we are reindexing, save the user metadata  */
-	if (private->reindex_on_shutdown) {
-		tracker_data_backup_save (private->ttl_backup_file, NULL);
-	}
-#endif
-}
-
-static void
-shutdown_directories (void)
-{
-	TrackerMainPrivate *private;
-
-	private = g_static_private_get (&private_key);
-
-	/* If we are reindexing, just remove the databases */
-	if (private->reindex_on_shutdown) {
-		tracker_db_manager_remove_all (FALSE);
-	}
-}
-
 static GStrv
 get_notifiable_classes (void)
 {
@@ -401,14 +359,6 @@ main (gint argc, gchar *argv[])
 	/* This makes sure we don't steal all the system's resources */
 	initialize_priority ();
 
-	/* Public locations */
-	private->ttl_backup_file =
-		g_build_filename (g_get_user_data_dir (),
-		                  "tracker",
-		                  "data",
-		                  "tracker-userdata-backup.ttl",
-		                  NULL);
-
 	/* Initialize major subsystems */
 	config = tracker_config_new ();
 	db_config = tracker_db_config_new ();
@@ -425,8 +375,6 @@ main (gint argc, gchar *argv[])
 		config_verbosity_changed_cb (G_OBJECT (config), NULL, NULL);
 	}
 
-	initialize_directories ();
-
 	if (!tracker_dbus_init ()) {
 		g_object_unref (db_config);
 		return EXIT_FAILURE;
@@ -550,9 +498,6 @@ main (gint argc, gchar *argv[])
 
 	g_message ("Cleaning up");
 
-	shutdown_databases ();
-	shutdown_directories ();
-
 	/* Shutdown major subsystems */
 	tracker_writeback_shutdown ();
 	tracker_events_shutdown ();
