@@ -51,7 +51,7 @@ G_DEFINE_TYPE (TrackerSteroids, tracker_steroids, G_TYPE_OBJECT)
 typedef struct {
 	DBusMessage *call_message;
 	int fd;
-	guint request_id;
+	TrackerDBusRequest *request;
 	DBusConnection *connection;
 	struct {
 		int query_count;
@@ -104,10 +104,7 @@ query_callback (gpointer  inthread_data,
 
 	if (ptr && ptr->error) {
 		/* Client is still there, but query failed */
-		tracker_dbus_request_failed (info->request_id,
-		                             NULL,
-		                             &ptr->error,
-		                             NULL);
+		tracker_dbus_request_end (info->request, ptr->error);
 		reply = dbus_message_new_error (info->call_message,
 		                                TRACKER_SPARQL_DBUS_ERROR,
 		                                ptr->error->message);
@@ -116,10 +113,7 @@ query_callback (gpointer  inthread_data,
 		g_error_free (ptr->error);
 	} else if (error) {
 		/* Client has disappeared */
-		tracker_dbus_request_failed (info->request_id,
-		                             NULL,
-		                             &error,
-		                             NULL);
+		tracker_dbus_request_end (info->request, error);
 		reply = dbus_message_new_error (info->call_message,
 		                                TRACKER_SPARQL_DBUS_ERROR,
 		                                error->message);
@@ -130,8 +124,7 @@ query_callback (gpointer  inthread_data,
 		DBusMessageIter iter, subiter;
 		guint i;
 
-		tracker_dbus_request_success (info->request_id,
-		                              NULL);
+		tracker_dbus_request_end (info->request, NULL);
 		reply = dbus_message_new_method_return (info->call_message);
 
 		dbus_message_iter_init_append (reply, &iter);
@@ -162,10 +155,7 @@ update_callback (GError *error, gpointer user_data)
 	DBusMessage *reply;
 
 	if (error) {
-		tracker_dbus_request_failed (info->request_id,
-		                             NULL,
-		                             &error,
-		                             NULL);
+		tracker_dbus_request_end (info->request, error);
 		reply = dbus_message_new_error (info->call_message,
 		                                TRACKER_SPARQL_DBUS_ERROR,
 		                                error->message);
@@ -174,7 +164,7 @@ update_callback (GError *error, gpointer user_data)
 		return;
 	}
 
-	tracker_dbus_request_success (info->request_id, NULL);
+	tracker_dbus_request_end (info->request, NULL);
 	reply = dbus_message_new_method_return (info->call_message);
 	dbus_connection_send (info->connection, reply, NULL);
 	dbus_message_unref (reply);
@@ -202,7 +192,7 @@ update_array_callback (GError *error, gpointer user_data)
 		guint i;
 		DBusMessageIter iter, subiter;
 
-		tracker_dbus_request_success (info->request_id, NULL);
+		tracker_dbus_request_end (info->request, NULL);
 		reply = dbus_message_new_method_return (info->call_message);
 
 		dbus_message_iter_init_append (reply, &iter);
@@ -277,10 +267,7 @@ update_blank_callback (GPtrArray *blank_nodes,
 	info = user_data;
 
 	if (error) {
-		tracker_dbus_request_failed (info->request_id,
-		                             NULL,
-		                             &error,
-		                             NULL);
+		tracker_dbus_request_end (info->request, error);
 		reply = dbus_message_new_error (info->call_message,
 		                                TRACKER_SPARQL_DBUS_ERROR,
 		                                error->message);
@@ -289,7 +276,7 @@ update_blank_callback (GPtrArray *blank_nodes,
 		return;
 	}
 
-	tracker_dbus_request_success (info->request_id, NULL);
+	tracker_dbus_request_end (info->request, NULL);
 	reply = dbus_message_new_method_return (info->call_message);
 	dbus_message_iter_init_append (reply, &iter);
 
@@ -463,7 +450,7 @@ steroids_query (TrackerSteroids *steroids,
                 DBusMessage     *message)
 {
 	ClientInfo *info;
-	guint request_id;
+	TrackerDBusRequest *request;
 	const gchar *sender;
 	const gchar *expected_signature;
 	DBusMessage *reply;
@@ -471,16 +458,9 @@ steroids_query (TrackerSteroids *steroids,
 	gchar *query;
 	int fd;
 
-	request_id = tracker_dbus_get_next_request_id ();
-
 	expected_signature = DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_UNIX_FD_AS_STRING;
 
 	if (g_strcmp0 (dbus_message_get_signature (message), expected_signature)) {
-		tracker_dbus_request_new (request_id,
-		                          NULL,
-		                          "%s()",
-		                          __FUNCTION__);
-
 		reply = dbus_message_new_error_printf (message,
 		                                       DBUS_ERROR_UNKNOWN_METHOD,
 		                                       UNKNOWN_METHOD_MESSAGE,
@@ -490,15 +470,6 @@ steroids_query (TrackerSteroids *steroids,
 		                                       expected_signature);
 		dbus_connection_send (connection, reply, NULL);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             UNKNOWN_METHOD_MESSAGE,
-		                             "Query",
-		                             dbus_message_get_signature (message),
-		                             dbus_message_get_interface (message),
-		                             expected_signature);
-
 		dbus_message_unref (reply);
 
 		return;
@@ -513,39 +484,28 @@ steroids_query (TrackerSteroids *steroids,
 	                       DBUS_TYPE_INVALID);
 
 	if (dbus_error_is_set (&dbus_error)) {
-		tracker_dbus_request_new (request_id,
-		                          NULL,
-		                          "%s()",
-		                          __FUNCTION__);
-
 		reply = dbus_message_new_error (message, dbus_error.name, dbus_error.message);
 		dbus_connection_send (connection, reply, NULL);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             dbus_error.message);
-
 		dbus_message_unref (reply);
 		dbus_error_free (&dbus_error);
 
 		return;
 	}
 
+	sender = dbus_message_get_sender (message);
+
+	request = tracker_dbus_request_begin (sender,
+	                                      "%s(query:'%s')",
+	                                      __FUNCTION__,
+	                                      query);
+
 	info = g_slice_new0 (ClientInfo);
 	info->connection = dbus_connection_ref (connection);
 	info->call_message = dbus_message_ref (message);
-	info->request_id = request_id;
+	info->request = request;
 	info->fd = fd;
 
-	tracker_dbus_request_new (request_id,
-	                          NULL,
-	                          "%s(query:'%s')",
-	                          __FUNCTION__,
-	                          query);
-
-	sender = dbus_message_get_sender (message);
-
 	tracker_store_sparql_query (query,
 	                            TRACKER_STORE_PRIORITY_HIGH,
 	                            query_inthread,
@@ -568,21 +528,14 @@ steroids_update (TrackerSteroids *steroids,
 	GDataInputStream *data_input_stream;
 	GError *error = NULL;
 	gsize bytes_read;
-	guint request_id;
+	TrackerDBusRequest *request;
 	const gchar *sender;
 	int query_size;
 	DBusMessage *reply;
 	gchar *query;
 	int fd;
 
-	request_id = tracker_dbus_get_next_request_id ();
-
 	if (g_strcmp0 (dbus_message_get_signature (message), DBUS_TYPE_UNIX_FD_AS_STRING)) {
-		tracker_dbus_request_new (request_id,
-		                          NULL,
-		                          "%s()",
-		                          __FUNCTION__);
-
 		reply = dbus_message_new_error_printf (message,
 		                                       DBUS_ERROR_UNKNOWN_METHOD,
 		                                       UNKNOWN_METHOD_MESSAGE,
@@ -593,15 +546,6 @@ steroids_update (TrackerSteroids *steroids,
 		dbus_connection_send (connection, reply, NULL);
 		dbus_message_unref (reply);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             UNKNOWN_METHOD_MESSAGE,
-		                             "Update",
-		                             dbus_message_get_signature (message),
-		                             dbus_message_get_interface (message),
-		                             DBUS_TYPE_UNIX_FD_AS_STRING);
-
 		return;
 	}
 
@@ -613,39 +557,28 @@ steroids_update (TrackerSteroids *steroids,
 	                       DBUS_TYPE_INVALID);
 
 	if (dbus_error_is_set (&dbus_error)) {
-		tracker_dbus_request_new (request_id,
-		                          NULL,
-		                          "%s()",
-		                          __FUNCTION__);
-
 		reply = dbus_message_new_error (message, dbus_error.name, dbus_error.message);
 		dbus_connection_send (connection, reply, NULL);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             dbus_error.message);
-
 		dbus_message_unref (reply);
 		dbus_error_free (&dbus_error);
 
 		return;
 	}
 
-	tracker_dbus_request_new (request_id,
-	                          NULL,
-	                          "%s(fd:%d)",
-	                          __FUNCTION__,
-	                          fd);
+	sender = dbus_message_get_sender (message);
+
+	request = tracker_dbus_request_begin (sender,
+	                                      "%s(fd:%d)",
+	                                      __FUNCTION__,
+	                                      fd);
 
 	info = g_slice_new0 (ClientInfo);
 	info->connection = dbus_connection_ref (connection);
 	info->call_message = dbus_message_ref (message);
-	info->request_id = request_id;
+	info->request = request;
 	info->fd = fd;
 
-	sender = dbus_message_get_sender (message);
-
 	input_stream = g_unix_input_stream_new (info->fd, TRUE);
 	data_input_stream = g_data_input_stream_new (input_stream);
 	g_buffered_input_stream_set_buffer_size (G_BUFFERED_INPUT_STREAM (data_input_stream),
@@ -663,10 +596,7 @@ steroids_update (TrackerSteroids *steroids,
 		dbus_connection_send (connection, reply, NULL);
 		dbus_message_unref (reply);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             error->message);
+		tracker_dbus_request_end (request, error);
 
 		g_object_unref (data_input_stream);
 		g_object_unref (input_stream);
@@ -693,10 +623,7 @@ steroids_update (TrackerSteroids *steroids,
 		dbus_connection_send (connection, reply, NULL);
 		dbus_message_unref (reply);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             error->message);
+		tracker_dbus_request_end (request, error);
 
 		g_free (query);
 		g_object_unref (data_input_stream);
@@ -707,8 +634,7 @@ steroids_update (TrackerSteroids *steroids,
 		return;
 	}
 
-	tracker_dbus_request_debug (request_id,
-	                            NULL,
+	tracker_dbus_request_debug (request,
 	                            "query: '%s'",
 	                            query);
 
@@ -746,21 +672,14 @@ steroids_update_array (TrackerSteroids *steroids,
 	GInputStream *input_stream;
 	GDataInputStream *data_input_stream;
 	GError *error = NULL;
-	guint request_id;
+	TrackerDBusRequest *request;
 	const gchar *sender;
 	int i;
 	DBusMessage *reply;
 	int fd;
 	gchar **query_array;
 
-	request_id = tracker_dbus_get_next_request_id ();
-
 	if (g_strcmp0 (dbus_message_get_signature (message), DBUS_TYPE_UNIX_FD_AS_STRING)) {
-		tracker_dbus_request_new (request_id,
-		                          NULL,
-		                          "%s()",
-		                          __FUNCTION__);
-
 		reply = dbus_message_new_error_printf (message,
 		                                       DBUS_ERROR_UNKNOWN_METHOD,
 		                                       UNKNOWN_METHOD_MESSAGE,
@@ -771,15 +690,6 @@ steroids_update_array (TrackerSteroids *steroids,
 		dbus_connection_send (connection, reply, NULL);
 		dbus_message_unref (reply);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             UNKNOWN_METHOD_MESSAGE,
-		                             "Update",
-		                             dbus_message_get_signature (message),
-		                             dbus_message_get_interface (message),
-		                             DBUS_TYPE_UNIX_FD_AS_STRING);
-
 		return;
 	}
 
@@ -791,39 +701,28 @@ steroids_update_array (TrackerSteroids *steroids,
 	                       DBUS_TYPE_INVALID);
 
 	if (dbus_error_is_set (&dbus_error)) {
-		tracker_dbus_request_new (request_id,
-		                          NULL,
-		                          "%s()",
-		                          __FUNCTION__);
-
 		reply = dbus_message_new_error (message, dbus_error.name, dbus_error.message);
 		dbus_connection_send (connection, reply, NULL);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             dbus_error.message);
-
 		dbus_message_unref (reply);
 		dbus_error_free (&dbus_error);
 
 		return;
 	}
 
-	tracker_dbus_request_new (request_id,
-	                          NULL,
-	                          "%s(fd:%d)",
-	                          __FUNCTION__,
-	                          fd);
+	sender = dbus_message_get_sender (message);
+
+	request = tracker_dbus_request_begin (sender,
+	                                      "%s(fd:%d)",
+	                                      __FUNCTION__,
+	                                      fd);
 
 	info = g_slice_new0 (ClientInfo);
 	info->connection = dbus_connection_ref (connection);
 	info->call_message = dbus_message_ref (message);
-	info->request_id = request_id;
+	info->request = request;
 	info->fd = fd;
 
-	sender = dbus_message_get_sender (message);
-
 	input_stream = g_unix_input_stream_new (info->fd, TRUE);
 	data_input_stream = g_data_input_stream_new (input_stream);
 	g_data_input_stream_set_byte_order (data_input_stream, G_DATA_STREAM_BYTE_ORDER_HOST_ENDIAN);
@@ -841,10 +740,7 @@ steroids_update_array (TrackerSteroids *steroids,
 		dbus_connection_send (connection, reply, NULL);
 		dbus_message_unref (reply);
 
-		tracker_dbus_request_failed (request_id,
-		                             NULL,
-		                             NULL,
-		                             error->message);
+		tracker_dbus_request_end (request, error);
 
 		g_object_unref (data_input_stream);
 		g_object_unref (input_stream);
@@ -872,10 +768,7 @@ steroids_update_array (TrackerSteroids *steroids,
 			dbus_connection_send (connection, reply, NULL);
 			dbus_message_unref (reply);
 
-			tracker_dbus_request_failed (request_id,
-			                             NULL,
-			                             NULL,
-			                             error->message);
+			tracker_dbus_request_end (request, error);
 
 			g_strfreev (query_array);
 			g_object_unref (data_input_stream);
@@ -903,10 +796,7 @@ steroids_update_array (TrackerSteroids *steroids,
 			dbus_connection_send (connection, reply, NULL);
 			dbus_message_unref (reply);
 
-			tracker_dbus_request_failed (request_id,
-			                             NULL,
-			                             NULL,
-			                             error->message);
+			tracker_dbus_request_end (request, error);
 
 			g_strfreev (query_array);
 			g_object_unref (data_input_stream);
@@ -924,8 +814,7 @@ steroids_update_array (TrackerSteroids *steroids,
 
 	for (i = 0; query_array[i] != NULL; i++) {
 
-		tracker_dbus_request_debug (request_id,
-		                            NULL,
+		tracker_dbus_request_debug (request,
 		                            "query: '%s'",
 		                            query_array[i]);
 
